@@ -5,7 +5,9 @@ from decimal import Decimal
 from itertools import groupby
 from functools import partial
 
+from trytond.i18n import gettext
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.model.exceptions import AccessError
 from trytond.wizard import Wizard, StateView, StateTransition, Button
 from trytond.pyson import If, In, Eval, Bool
 from trytond.transaction import Transaction
@@ -36,13 +38,20 @@ class PurchaseRequest(ModelSQL, ModelView):
     quantity = fields.Float('Quantity', required=True, states=STATES,
         digits=(16, Eval('uom_digits', 2)), depends=DEPENDS + ['uom_digits'])
     uom = fields.Many2One('product.uom', 'UOM', select=True,
+        ondelete='RESTRICT',
+        domain=[
+            ('category', '=', Eval('product_uom_category')),
+            ],
         states={
             'required': Bool(Eval('product')),
             'readonly': STATES['readonly'],
             },
-        depends=['product'] + DEPENDS)
+        depends=['product', 'product_uom_category'] + DEPENDS)
     uom_digits = fields.Function(fields.Integer('UOM Digits'),
         'on_change_with_uom_digits')
+    product_uom_category = fields.Function(
+        fields.Many2One('product.uom.category', "Product Uom Category"),
+        'on_change_with_product_uom_category')
     computed_quantity = fields.Float('Computed Quantity', readonly=True)
     computed_uom = fields.Many2One('product.uom', 'Computed UOM',
         readonly=True)
@@ -85,12 +94,6 @@ class PurchaseRequest(ModelSQL, ModelView):
     def __setup__(cls):
         super(PurchaseRequest, cls).__setup__()
         cls._order[0] = ('id', 'DESC')
-        cls._error_messages.update({
-                'create_request': ('Purchase requests are only created '
-                    'by the system.'),
-                'delete_purchase_line': ('You can not delete purchased '
-                    'request.'),
-                })
         cls._buttons.update({
                 'handle_purchase_cancellation_exception': {
                     'invisible': Eval('state') != 'exception',
@@ -173,22 +176,14 @@ class PurchaseRequest(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        _, operator, value = clause
-        if operator.startswith('!') or operator.startswith('not'):
-            bool_op = 'AND'
-        else:
-            bool_op = 'OR'
-        domain = [bool_op]
-        if value is not None:
-            names = value.split('@', 1)
-            sub_domain = [('product.template.name', operator, names[0])]
-            if len(names) != 1 and names[1]:
-                sub_domain.append(('warehouse', operator, names[1]))
-                if bool_op == 'AND':
-                    sub_domain.insert(0, 'OR')
-            domain.append(sub_domain)
-        domain.append(('description', *clause[1:]))
-        return domain
+        res = []
+        names = clause[2].split('@', 1)
+        res.append(('product.template.name', clause[1], names[0]))
+        if len(names) != 1 and names[1]:
+            res.append(('warehouse', clause[1], names[1]))
+        return ['OR', res,
+            ('description',) + tuple(clause[1:]),
+            ]
 
     @staticmethod
     def default_company():
@@ -244,6 +239,11 @@ class PurchaseRequest(ModelSQL, ModelView):
             return self.uom.digits
         return 2
 
+    @fields.depends('product')
+    def on_change_with_product_uom_category(self, name=None):
+        if self.product:
+            return self.product.default_uom_category.id
+
     @fields.depends('product')
     def on_change_with_default_uom_digits(self, name=None):
         if self.product:
@@ -264,40 +264,58 @@ class PurchaseRequest(ModelSQL, ModelView):
                 ])
         return [(None, '')] + [(m.model, m.name) for m in models]
 
+    @classmethod
+    def view_attributes(cls):
+        return [
+            ('/tree', 'visual', If(Eval('state') == 'cancel', 'muted', '')),
+            ]
+
     @classmethod
     def create(cls, vlist):
         for vals in vlist:
             for field_name in ('quantity', 'company'):
                 if vals.get(field_name) is None:
-                    cls.raise_user_error('create_request')
+                    raise AccessError(
+                        gettext('purchase_request.msg_request_no_create'))
         return super(PurchaseRequest, cls).create(vlist)
 
     @classmethod
     def delete(cls, requests):
-        if any(r.purchase_line for r in requests):
-            cls.raise_user_error('delete_purchase_line')
+        for request in requests:
+            if request.purchase_line:
+                raise AccessError(
+                    gettext('purchase_request.msg_request_delete_purchased',
+                        request=request.rec_name))
         super(PurchaseRequest, cls).delete(requests)
 
     @classmethod
-    def find_best_supplier(cls, product, date):
+    def find_best_product_supplier(cls, product, date, **pattern):
+        "Return the best product supplier to request product at date"
+        pool = Pool()
+        Date = pool.get('ir.date')
+        today = Date.today()
+        for product_supplier in product.product_suppliers_used(**pattern):
+            supply_date = product_supplier.compute_supply_date(date=today)
+            timedelta = date - supply_date
+            if timedelta >= datetime.timedelta(0):
+                return product_supplier
+
+    @classmethod
+    def find_best_supplier(cls, product, date, **pattern):
         '''
         Return the best supplier and purchase_date for the product.
         '''
-        Date = Pool().get('ir.date')
+        pool = Pool()
+        Date = pool.get('ir.date')
 
-        supplier = None
-        today = Date.today()
-        for product_supplier in product.product_suppliers:
-            supply_date = product_supplier.compute_supply_date(date=today)
-            timedelta = date - supply_date
-            if not supplier and timedelta >= datetime.timedelta(0):
-                supplier = product_supplier.party
-                break
-
-        if supplier:
+        product_supplier = cls.find_best_product_supplier(
+            product, date, **pattern)
+        if product_supplier:
+            supplier = product_supplier.party
             purchase_date = product_supplier.compute_purchase_date(date)
         else:
-            purchase_date = today
+            supplier = None
+            purchase_date = Date.today()
         return supplier, purchase_date
 
     @classmethod
@@ -326,15 +344,6 @@ class CreatePurchase(Wizard):
             Button('Continue', 'start', 'tryton-forward', default=True),
             ])
 
-    @classmethod
-    def __setup__(cls):
-        super(CreatePurchase, cls).__setup__()
-        cls._error_messages.update({
-                'missing_price': 'Purchase price is missing for product "%s".',
-                'please_update': ('This price is necessary for creating '
-                    'purchases.'),
-                })
-
     def default_ask_party(self, fields):
         Request = Pool().get('purchase.request')
         requests = Request.browse(Transaction().context['active_ids'])
@@ -349,8 +358,8 @@ class CreatePurchase(Wizard):
             'company': request.company.id,
             }
 
-    @staticmethod
-    def _group_purchase_key(requests, request):
+    @classmethod
+    def _group_purchase_key(cls, requests, request):
         '''
         The key to group lines by purchases
         A list of key-value as tuples of the purchase
@@ -460,6 +469,10 @@ class CreatePurchase(Wizard):
         line.quantity = sum(r.quantity for r in requests)
         line.purchase = purchase
         line.on_change_product()
+        # Set again in case on_change's changed them
+        for f, v in key:
+            setattr(line, f, v)
+        line.on_change_quantity()
         return line
 
 
